diff --git a/src/virsh.c b/src/virsh.c
--- a/src/virsh.c
+++ b/src/virsh.c
@@ -1667,6 +1667,55 @@ cmdSetmem(vshControl * ctl, vshCmd * cmd
 cmdSetmem(vshControl * ctl, vshCmd * cmd)
 {
     virDomainPtr dom;
+    virDomainInfo info;
+    int kilobytes;
+    int ret = TRUE;
+
+    if (!vshConnectionUsability(ctl, ctl->conn, TRUE))
+        return FALSE;
+
+    if (!(dom = vshCommandOptDomain(ctl, cmd, "domain", NULL)))
+        return FALSE;
+
+    if (virDomainGetInfo(dom, &info) != 0) {
+	info.maxMem = 0;
+    }
+    kilobytes = vshCommandOptInt(cmd, "kilobytes", &kilobytes);
+    if ((kilobytes <= 0)  || (kilobytes > info.maxMem)) {
+        virDomainFree(dom);
+        vshError(ctl, FALSE, _("Invalid value of %d for memory size"), kilobytes);
+        return FALSE;
+    }
+
+    if (virDomainSetMemory(dom, kilobytes) != 0) {
+        ret = FALSE;
+    }
+
+    virDomainFree(dom);
+    return ret;
+}
+
+/*
+ * "setmaxmem" command
+ */
+static vshCmdInfo info_setmaxmem[] = {
+    {"syntax", "setmaxmem <domain> <kilobytes>"},
+    {"help", gettext_noop("change maximum memory limit")},
+    {"desc", gettext_noop("Change the maximum memory allocation limit in the guest domain.")},
+    {NULL, NULL}
+};
+
+static vshCmdOptDef opts_setmaxmem[] = {
+    {"domain", VSH_OT_DATA, VSH_OFLAG_REQ, gettext_noop("domain name, id or uuid")},
+    {"kilobytes", VSH_OT_DATA, VSH_OFLAG_REQ, gettext_noop("maximum memory limit in kilobytes")},
+    {NULL, 0, 0, NULL}
+};
+
+static int
+cmdSetmaxmem(vshControl * ctl, vshCmd * cmd)
+{
+    virDomainPtr dom;
+    virDomainInfo info;
     int kilobytes;
     int ret = TRUE;
 
@@ -1682,49 +1731,14 @@ cmdSetmem(vshControl * ctl, vshCmd * cmd
         vshError(ctl, FALSE, _("Invalid value of %d for memory size"), kilobytes);
         return FALSE;
     }
-
-    if (virDomainSetMemory(dom, kilobytes) != 0) {
-        ret = FALSE;
-    }
-
-    virDomainFree(dom);
-    return ret;
-}
-
-/*
- * "setmaxmem" command
- */
-static vshCmdInfo info_setmaxmem[] = {
-    {"syntax", "setmaxmem <domain> <kilobytes>"},
-    {"help", gettext_noop("change maximum memory limit")},
-    {"desc", gettext_noop("Change the maximum memory allocation limit in the guest domain.")},
-    {NULL, NULL}
-};
-
-static vshCmdOptDef opts_setmaxmem[] = {
-    {"domain", VSH_OT_DATA, VSH_OFLAG_REQ, gettext_noop("domain name, id or uuid")},
-    {"kilobytes", VSH_OT_DATA, VSH_OFLAG_REQ, gettext_noop("maximum memory limit in kilobytes")},
-    {NULL, 0, 0, NULL}
-};
-
-static int
-cmdSetmaxmem(vshControl * ctl, vshCmd * cmd)
-{
-    virDomainPtr dom;
-    int kilobytes;
-    int ret = TRUE;
-
-    if (!vshConnectionUsability(ctl, ctl->conn, TRUE))
-        return FALSE;
-
-    if (!(dom = vshCommandOptDomain(ctl, cmd, "domain", NULL)))
-        return FALSE;
-
-    kilobytes = vshCommandOptInt(cmd, "kilobytes", &kilobytes);
-    if (kilobytes <= 0) {
-        virDomainFree(dom);
-        vshError(ctl, FALSE, _("Invalid value of %d for memory size"), kilobytes);
-        return FALSE;
+    if (virDomainGetInfo(dom, &info) != 0) {
+	info.memory = 0x7fffffff;
+    }
+
+    if (kilobytes < info.memory) {
+       if (virDomainSetMemory(dom, kilobytes) != 0) {
+          return FALSE;
+       }
     }
 
     if (virDomainSetMaxMemory(dom, kilobytes) != 0) {
